package com.persagy.apm.energy.report.common.utils;

import org.nfunk.jep.JEP;
import org.springframework.util.CollectionUtils;

import java.lang.reflect.Field;
import java.math.BigDecimal;
import java.util.List;
import java.util.stream.Collectors;

public class DataUtils {
    /**
     * 对象转Double类型
     *
     * @param object
     * @return
     */
    public static Double parseDouble(Object object) {
        if (object == null) {
            return null;
        }
        if (object instanceof String) {
            return Double.valueOf(object.toString());
        }
        if (object instanceof Integer) {
            return ((Integer) object).doubleValue();
        }
        if (object instanceof Long) {
            return ((Long) object).doubleValue();
        }
        if (object instanceof Double) {
            return (Double) object;
        }
        if (object instanceof BigDecimal) {
            return ((BigDecimal) object).doubleValue();
        }
        return (Double) object;
    }

    /**
     * 按规则进行计算
     *
     * @param standardValue  规则值
     * @param qualifyFormula 规则
     * @param data           要进行规则校验的数据
     * @return
     */
    public static Double getQualifyResult(Double standardValue, String qualifyFormula, Double data) {
        JEP jep = new JEP();
        // 添加常用函数
        jep.addStandardFunctions();
        // 添加常用常量
        jep.addStandardConstants();
        jep.addVariable("standard_value", standardValue);
        jep.addVariable("source", data);
        jep.parseExpression(qualifyFormula);
        return jep.getValue();
    }


    /**
     * 将一组对象中类型为double的属性求和
     *
     * @param objs   要求和的一组对象
     * @param tClass 这组对象的类型
     * @return 一个新的对象，将求和之后的属性赋值给它
     * @author lixing
     * @version V1.0 2021/5/30 10:44 上午
     */
    public static <T> T sumDoubleParams(List<T> objs, Class tClass) {
        try {
            T result = (T) tClass.newInstance();
            if (CollectionUtils.isEmpty(objs)) {
                return result;
            }
            Field[] declaredFields = tClass.getDeclaredFields();
            for (Field declaredField : declaredFields) {
                if (!Double.class.equals(declaredField.getType())) {
                    continue;
                }
                List<T> notNullList = objs.stream().filter(
                        obj -> {
                            try {
                                return declaredField.get(obj) != null;
                            } catch (IllegalAccessException e) {
                                e.printStackTrace();
                                return false;
                            }
                        }).collect(Collectors.toList());
                if (CollectionUtils.isEmpty(notNullList)) {
                    declaredField.set(result, null);
                } else {
                    declaredField.set(result, notNullList.stream().mapToDouble(
                            obj -> {
                                try {
                                    return (Double) declaredField.get(obj);
                                } catch (IllegalAccessException e) {
                                    e.printStackTrace();
                                    return 0d;
                                }
                            }).sum());
                }

            }
            return result;
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }
}
